# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from .card_header import CardHeader
from .card import Card
from .section import Section
from .widgets.text_paragraph import TextParagraph


class CardTest(unittest.TestCase):
  def test_simple_render(self) -> None:
    self.maxDiff = None
    header = CardHeader(title='Princess Bride')
    section = Section()
    section.add_widget(TextParagraph(text="Inconceivable!"))
    card = Card()
    card.card_id = 'vizzini'
    card.header = header
    card.add_section(section)
    output = card.render()

    print(output)

    self.assertDictEqual(
        output,
        {
            'cardId': 'vizzini',
            'card': {
                'header': {'title': 'Princess Bride'},
                'sections': [{
                    'widgets': [
                      {'textParagraph': {
                          'text': 'Inconceivable!'
                      }}]}]}})

  def test_render_no_header(self) -> None:
    self.maxDiff = None
    section = Section()
    section.add_widget(TextParagraph(text="Inconceivable!"))
    card = Card()
    card.card_id = 'vizzini'
    card.add_section(section)
    output = card.render()

    print(output)

    self.assertDictEqual(
        output,
        {
            'cardId': 'vizzini',
            'card': {
                'sections': [{
                    'widgets': [
                      {'textParagraph': {
                          'text': 'Inconceivable!'
                      }}]}]}})



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

import dataclasses
from typing import Any, List, Mapping

import dataclasses_json
from card_framework import list_field, standard_field

from .action_response import ActionResponse
from .annotation import Annotation
from .attachment import Attachment
from .card import Card
from .space import Space
from .user import User


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class Message(object):
  name: str = standard_field()
  sender: User = standard_field()
  create_time: str = standard_field()
  last_update_time: str = standard_field()
  text: str = standard_field()
  cards: List[Card] = list_field(field_name='cardsV2')
  annotations: List[Annotation] = list_field()
  thread: Thread = standard_field()
  space: Space = standard_field()
  fallback_text: str = standard_field()
  action_response: ActionResponse = standard_field()
  argument_text: str = standard_field()
  slashCommand: SlashCommand = standard_field()
  attachment: List[Attachment] = list_field()
  matchedUrl: MatchedUrl = standard_field()

  def render(self, remove_empty_strings: bool = False) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return self.to_dict()


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class Thread(object):
  name: str = standard_field()


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class SlashCommand(object):
  commandId: str = standard_field()


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class MatchedUrl(object):
  url: str = standard_field()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from .icon import Icon
from ..enums import Icon as KnownIcon


class IconTest(unittest.TestCase):
  def test_icon_icon_url(self) -> None:
    i = Icon(
        icon_url='https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')

    self.assertDictEqual(
        i.to_dict(),
        {
            'iconUrl': 'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg',
        })

  def test_icon_to_url(self) -> None:
    i = Icon(
        icon_url='https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')
    self.assertIsNone(i.known_icon)

    i.known_icon = KnownIcon.AIRPLANE
    self.assertIsNone(i.icon_url)
    self.assertEqual(
        i.to_dict(),
        {

            'knownIcon': 'AIRPLANE',
        })

  def test_url_to_icon(self) -> None:
    i = Icon(known_icon=KnownIcon.AIRPLANE)
    self.assertIsNone(i.icon_url)

    i.icon_url = 'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg'
    self.assertIsNone(i.known_icon)
    self.assertEqual(
        i.to_dict(),
        {
            'iconUrl': 'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg',
        })

  def test_must_have_one(self) -> None:
    i = Icon()

    with self.assertRaisesRegex(ValueError, 'One of .* must be set\.'):
      i.to_dict()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

import dataclasses
from typing import Any, Dict, Mapping

import card_framework
import dataclasses_json
from dataclasses_json.core import Json

from .enums import AnnotationType, AutoNumber
from .user import User


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class Annotation(object):
  type: AnnotationType = card_framework.enum_field()
  startIndex: int = card_framework.standard_field()
  length: int = card_framework.standard_field()
  user_mention: UserMentionMetadata = card_framework.standard_field()
  slash_command: SlashCommandMetadata = card_framework.standard_field()

  def __setattr__(self, __name: str, __value: Any) -> None:
    """Sets attributes.

    This is overridden to ensure that one and only one of action and open_link
    can be set. If an attempt is made to set both, the already set one will be
    automatically cleared back to 'None'.

    If an attempt is being made to set a value to 'None', then no checks are
    performed, the value is just passed on.

    Args:
        __name (str): The name of the property to set.
        __value (Any): The value to set the property to.

    Returns:
        _type_: _description_
    """
    if __value:
      if __name == 'user_mention':
        self.slash_command = None
      elif __name == 'slash_command':
        self.user_mention = None

    super().__setattr__(__name, __value)

  def to_dict(self, encode_json=False) -> Dict[str, Json]:
    """Converts the dataclass to a dict.

    This is an override of the standard dataclass `to_dict` method to allow
    validation that `action` and `open_link` are mutually exclusive.

    Args:
        encode_json (bool, optional): encode the json strings. Defaults to False.

    Raises:
        ValueError: if both `known_icon` and `icon_url` are set.

    Returns:
        Dict[str, Json]: The header
    """
    if all([self.slash_command, self.user_mention]):
      raise ValueError('Only one of [slash_command, user_mention] can be set.')
    elif not(any([self.slash_command, self.user_mention])):
      raise ValueError(f'One of [slash_command, user_mention] must be set.')

    return super().to_dict(encode_json)

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return {'annotation': self.to_dict(), }


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class UserMentionMetadata(object):
  class UserMentionMetadataType(AutoNumber):
    TYPE_UNSPECIFIED = 'TYPE_UNSPECIFIED'
    ADD = 'ADD'
    MENTION = 'MENTION'

  user: User = card_framework.standard_field()
  type: UserMentionMetadataType = card_framework.enum_field()

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return self.to_dict()


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class SlashCommandMetadata(object):
  class SlashCommandMetadataType(AutoNumber):
    TYPE_UNSPECIFIED = 'TYPE_UNSPECIFIED'
    ADD = 'ADD'
    INVOKE = 'INVOKE'

  bot: User = card_framework.standard_field()
  type: SlashCommandMetadataType = card_framework.enum_field()
  command_name: str = card_framework.standard_field()
  command_id: str = card_framework.standard_field()
  triggers_dialog: bool = card_framework.standard_field()

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return self.to_dict()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass, field
from typing import Any, Dict, List, Mapping, Optional

from dataclasses_json import DataClassJsonMixin, config
from dataclasses_json.core import Json

from card_framework import list_field, standard_field

from .widget import Widget


@dataclass
class Section(DataClassJsonMixin):
  """Section

  Describes a Google Chat App response Section.

  https://developers.google.com/chat/api/guides/message-formats/cards#sections_and_widgets
  """

  # def render_widgets(self, x) -> Dict[str, Json]:
  #   return [widget.render() for widget in x]

  header: Optional[str] = standard_field()
  widgets: Optional[List[Widget]] = list_field(default_factory=list)
  collapsible: bool = standard_field()
  uncollapsible_widgets_count: int = standard_field()

  def add_widget(self, widget: Widget) -> None:
    """Adds a widget to the section.

    Widgets need to be added in a specific way, including the tag. Simply adding
    a widget object to the `List` of widgets is not sufficient. As a result this
    helper method should be used for safety. This uses the built in widget's
    attribute `_widget_tag` which will ensure the widget is rendered correctly
    when necessary.

    Args:
        widget (Widget): The widget to be added to the section.
    """
    self.widgets.append(widget)

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return self.to_dict()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from .image import Image
from .on_click import OnClick, OpenLink


class ImageTest(unittest.TestCase):
  def test_image_url_only(self) -> None:
    i = Image(
        image_url='https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')

    self.assertEqual(i._widget_tag, 'image')
    self.assertEqual(i.image_url,
                     'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')
    self.assertIsNone(i.on_click)
    self.assertDictEqual({
        'imageUrl':
        'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg',
    },
        i.to_dict())

  def test_image_url_with_on_click(self) -> None:
    i = Image(
        image_url='https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')

    on_click = OnClick(open_link=OpenLink(url='https://www.karentaylorart.com'))
    i.on_click = on_click

    self.assertEqual(i.image_url,
                     'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')
    self.assertEqual(i.on_click, on_click)
    self.assertEqual(
        i.to_dict(),
        {'imageUrl':
         'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg',
         'onClick': {'openLink': {'url': 'https://www.karentaylorart.com'}}})



# Copyright 2022 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations
import inspect
from typing import Any, Callable, Mapping

import dataclasses_json
import dataclasses


def lazy_property(f: Callable):
  """Decorator that makes a property lazy-evaluated.

  Args:
    f: the function to convert to a lazy property.
  """
  attr_name = '_lazy_' + f.__name__

  @property
  def _lazy_property(self) -> Any:
    if not hasattr(self, attr_name):
      setattr(self, attr_name, f(self))
    return getattr(self, attr_name)
  return _lazy_property


def field(default: Any = None, default_factory: Any = None,
          **metadata) -> dataclasses.Field:
  return \
      dataclasses.field(default_factory=default_factory,
                        metadata=dataclasses_json.config(**metadata)) \
      if default_factory else \
      dataclasses.field(default=default,
                        metadata=dataclasses_json.config(**metadata))


def metadata(base: Mapping[str, Any], **custom) -> Mapping[str, Any]:
  for key in custom:
    if not custom[key] and key in base:
      del base[key]

    elif custom[key]:
      base.update({key: custom[key]})

  return base


def standard_field(default: Any = None, default_factory: Any = None,
                   **kwargs) -> dataclasses.Field:
  base = {
      'letter_case': dataclasses_json.LetterCase.CAMEL,
      'exclude': lambda x: not x
  }

  return field(default=default, default_factory=default_factory,
               **metadata(base=base, **kwargs))


def enum_field(default: Any = None, **kwargs) -> dataclasses.Field:
  base = {
      'encoder': lambda x: x.name if x else None
  }

  return standard_field(default=default, **base, **kwargs)


def list_field(default: Any = None, default_factory: Any = list,
               **kwargs) -> dataclasses.Field:
  base = {
      'encoder': lambda x: [
          f.render() if inspect.getmembers(
              f,
              lambda m: inspect.ismethod(m) and m.__name__ == 'render'
          ) else f.to_dict() for f in x],

  }

  return standard_field(default_factory=list, **base, **kwargs)



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from dataclasses import dataclass
import unittest

from dataclasses_json import LetterCase, dataclass_json

from .enums import HorizontalAlignment

from .section import Section
from .widget import Widget


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TestWidget(Widget):
  """Good Widget"""
  text: str = ''

  @property
  def _widget_tag(self) -> str:
    return 'testWidget'


class SectionTest(unittest.TestCase):
  def test_empty(self) -> None:
    section = Section()

    self.assertEqual(list(), section.widgets)
    self.assertEqual(None, section.header)
    self.assertDictEqual(
        section.to_dict(),
        {})

  def test_fully_specified(self) -> None:
    header = 'Princess Bride'
    widget = TestWidget(text="You keep using that word...")

    section = Section()
    section.header = header
    section.add_widget(widget)

    self.assertEqual(1, len(section.widgets))
    self.assertEqual({
        'header': 'Princess Bride',
        'widgets': [{
            'testWidget': {'text': 'You keep using that word...'}}, ],
    },
        section.to_dict())

  def test_header_no_widgets(self) -> None:
    header = 'Princess Bride'

    section = Section()
    section.header = header
    self.assertEqual(list(), section.widgets)
    self.assertEqual({
        'header': 'Princess Bride',
    }, section.to_dict())

  def test_widgets_no_header(self) -> None:
    widget = TestWidget(text="You keep using that word...")

    section = Section()
    section.add_widget(widget)

    self.assertEqual(1, len(section.widgets))
    self.assertDictEqual(section.to_dict(),
                         {
        'widgets': [{
            'testWidget': {'text': 'You keep using that word...'}}, ], }
    )

  def test_multiple_widgets(self) -> None:
    header = 'Princess Bride'
    section = Section()
    section.header = header
    section.add_widget(TestWidget(text="You keep using that word..."))
    section.add_widget(TestWidget(text="Inconceivable!"))

    self.assertEqual(2, len(section.widgets))
    self.assertEqual({
        'header': 'Princess Bride',
        'widgets': [{
            'testWidget': {'text': 'You keep using that word...'}}, {
            'testWidget': {'text': 'Inconceivable!'}}],
    },
        section.to_dict())

  def test_aligned_widget(self) -> None:
    widget = TestWidget(text="You keep using that word...")
    widget.horizontal_alignment = HorizontalAlignment.CENTER
    widget.to_dict()

    section = Section()
    section.add_widget(widget)

    self.assertEqual(1, len(section.widgets))
    self.assertDictEqual({
        'widgets': [{
            'horizontalAlignment': 'CENTER',
            'testWidget': {'text': 'You keep using that word...'}}, ],
    },
        section.to_dict())



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass, field
from typing import Any, Optional

from dataclasses_json import LetterCase, config, dataclass_json

from card_framework import standard_field


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Color(object):
  red: Optional[float] = standard_field()
  green: Optional[float] = standard_field()
  blue: Optional[float] = standard_field()
  alpha: Optional[float] = standard_field()

  def __setattr__(self, __name: str, __value: Any) -> None:
    """Sets attributes.

    This is overridden to ensure that one and only one of action and open_link
    can be set. If an attempt is made to set both, the already set one will be
    automatically cleared back to 'None'.

    If an attempt is being made to set a value to 'None', then no checks are
    performed, the value is just passed on.

    Args:
        __name (str): The name of the property to set.
        __value (Any): The value to set the property to.

    Returns:
        _type_: _description_
    """
    if __value:
      if __value < 0 or __value > 1:
        raise ValueError('Color or alpha values must be between 0 and 1.')

    super().__setattr__(__name, __value)



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json

from card_framework import standard_field
from .widgets.button import Button


@dataclass_json
@dataclass
class CardFixedFooter(object):
  primary_button: Optional[Button] = standard_field()
  secondary_button: Optional[Button] = standard_field()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from ..enums import HorizontalAlignment

from .text_paragraph import TextParagraph


class TextParagraphTest(unittest.TestCase):
  def test_simple_render(self) -> None:
    self.assertDictEqual(TextParagraph(text='Foo!').render(),
                         {'textParagraph': {'text': 'Foo!'}})

  def test_aligned(self) -> None:
    text_paragraph = TextParagraph(text='Foo!')
    text_paragraph.horizontal_alignment = HorizontalAlignment.CENTER

    self.assertDictEqual(text_paragraph.render(),
                         {'textParagraph': {'text': 'Foo!'},
                          'horizontalAlignment': 'CENTER'})



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

import dataclasses
from typing import Any, Dict, Iterable, Mapping

from  card_framework import enum_field, standard_field
import dataclasses_json
from dataclasses_json.core import Json

from .enums import SpaceType


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class Space(object):
  name: str = standard_field()
  type: str = standard_field() # deprecated
  spaceType: SpaceType = enum_field()
  singleUserBotDm: bool = standard_field()
  threaded: bool = standard_field()
  displayName: str = standard_field()
  spaceDetails: SpaceDetail = standard_field()


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class SpaceDetail(object):
  description: str = standard_field()
  guidelines: str = standard_field()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from .color import Color


class ColorTest(unittest.TestCase):
  def test_valid_red(self) -> None:
    c = Color(red=1)
    self.assertDictEqual(
        c.to_dict(),
        {'red': 1})

  def test_valid_green(self) -> None:
    c = Color(green=0.5)
    self.assertDictEqual(
        c.to_dict(),
        {'green': 0.5})

  def test_valid_purple(self) -> None:
    c = Color(red=1, blue=0.5)
    self.assertDictEqual(
        c.to_dict(),
        {'red': 1, 'blue': 0.5})

  def test_invalid_number(self) -> None:
    with self.assertRaisesRegex(
            ValueError,
            'Color or alpha values must be between 0 and 1.'):
      c = Color(red=1.5)



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, List, Mapping

from card_framework import list_field, standard_field
from dataclasses_json import LetterCase, config, dataclass_json

from ..enums import LoadIndicator
from ..widget import Widget


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Action(Widget):
  function: str = standard_field(default='')
  parameters: List[ActionParameter] = list_field()
  load_indicator: LoadIndicator = standard_field()
  persist_values: bool = standard_field()

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    return 'action'


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ActionParameter(Widget):
  key: str = standard_field()
  value: str = standard_field()

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    return 'actionParameter'

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return self.to_dict()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import inspect
from dataclasses import dataclass, field
from typing import Any, Mapping, Optional

import dataclasses_json
import stringcase
from card_framework import standard_field
from dataclasses_json.core import Json

from .enums import HorizontalAlignment


@dataclass
class Widget(dataclasses_json.DataClassJsonMixin):
  """Widget

  Parent class to all widgets. Each child Widget must implement the
  `_widget_tag` property which is the key the Section.widgets list will name
  the rendered widget map.
  """
  _horizontal_alignment: HorizontalAlignment = \
      standard_field(exclude=lambda x: True)
  # field(
  #     default=None, metadata=dataclasses_json.config(
  #         exclude=lambda x: not False))

  @property
  def horizontal_alignment(self) -> str:
    return \
        self._horizontal_alignment.name if self._horizontal_alignment else None

  @horizontal_alignment.setter
  def horizontal_alignment(self, a: HorizontalAlignment = None) -> None:
    self._horizontal_alignment = a

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Raises:
        NotImplementedError: If not implemented.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    raise NotImplementedError('All widgets must provide their tag name.')

  def render(self) -> Mapping[str, Any]:
    """Renders the widget in a usable form.

    Returns:
        Mapping[str, Any]: the json representation of the widget
    """
    render = {}
    properties = inspect.getmembers(self.__class__,
                                    lambda v: isinstance(v, property))
    for (name, value) in properties:
      if name == '_widget_tag':
        render[self._widget_tag] = self.to_dict()
      else:
        if widget_value := value.fget(self):
          render[stringcase.camelcase(name)] = widget_value

    return render



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from .action import Action, ActionParameter
from .on_click import OnClick
from .open_link import OpenLink


class OnClickTest(unittest.TestCase):
  def test_on_click_empty(self) -> None:
    o = OnClick()
    self.assertEqual(o.action, None)
    self.assertEqual(o.open_link, None)
    self.assertEqual({}, o.to_dict())

  def test_on_click_action(self) -> None:
    o = OnClick()
    o.action = Action(function='method', parameters=[
        ActionParameter(key='key1', value='value1'),
        ActionParameter(key='key2', value='value2'),
    ])
    self.assertNotEqual(o.action, None)
    self.assertEqual(o.open_link, None)
    self.assertEqual({'action': {'function': 'method',
                                 'parameters': [
                                     {'key': 'key1', 'value': 'value1'},
                                     {'key': 'key2', 'value': 'value2'}]}},
                     o.to_dict())

  def test_on_click_open_link(self) -> None:
    o = OnClick()
    o.open_link = OpenLink(url='https://www.karentaylorart.com')
    self.assertEqual(o.action, None)
    self.assertNotEqual(o.open_link, None)
    self.assertEqual({'openLink': {'url': 'https://www.karentaylorart.com'}},
                     o.to_dict())

  def test_on_click_action_after_open_link(self) -> None:
    o = OnClick()
    o.open_link = OpenLink(url='https://www.karentaylorart.com')

    self.assertEqual(o.action, None)
    self.assertNotEqual(o.open_link, None)

    o.action = Action(function='method', parameters=[
        ActionParameter(key='key1', value='value1'),
        ActionParameter(key='key2', value='value2'),
    ])

    self.assertNotEqual(o.action, None)
    self.assertEqual(o.open_link, None)
    self.assertEqual({'action': {'function': 'method',
                                 'parameters': [
                                     {'key': 'key1', 'value': 'value1'},
                                     {'key': 'key2', 'value': 'value2'}]}},
                     o.to_dict())

  def test_on_click_open_link_after_action(self) -> None:
    o = OnClick()
    o.action = Action(function='method', parameters=[
        ActionParameter(key='key1', value='value1'),
        ActionParameter(key='key2', value='value2'),
    ])

    self.assertNotEqual(o.action, None)
    self.assertEqual(o.open_link, None)

    o.open_link = OpenLink(url='https://www.karentaylorart.com')

    self.assertEqual(o.action, None)
    self.assertNotEqual(o.open_link, None)
    self.assertEqual({'openLink': {'url': 'https://www.karentaylorart.com'}},
                     o.to_dict())



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from card_framework import *
from card_framework.v2.enums import AutoNumber
from dataclasses import dataclass
from typing import Any, List, Mapping

from card_framework import list_field, standard_field
from dataclasses_json import dataclass_json, LetterCase


class LazyPropertyTest(unittest.TestCase):
  class Foo(object):
    @lazy_property
    def lazy_thing(self) -> str:
      return 'lazy'

  def test_lazy_thing(self):
    foo = LazyPropertyTest.Foo()
    self.assertFalse(hasattr(foo, '_lazy_lazy_thing'))
    self.assertEqual('lazy', foo.lazy_thing)
    self.assertTrue(hasattr(foo, '_lazy_lazy_thing'))


class TestEnum(AutoNumber):
  ONE = 'ONE'
  TWO = 'TWO'


class EnumFieldTest(unittest.TestCase):
  @dataclass_json
  @dataclass
  class TestClass(object):
    field_enum: TestEnum = enum_field()

  def test_no_default(self) -> None:
    field = enum_field()

    self.assertIsNone(field.default)

  def test_default(self) -> None:
    field = enum_field(TestEnum.ONE)

    self.assertEqual(field.default, TestEnum.ONE)

  def test_class_render_empty(self) -> None:
    c = self.TestClass()

    self.assertDictEqual(c.to_dict(), {})

  def test_class_render_populated(self) -> None:
    c = self.TestClass()
    c.field_enum = TestEnum.ONE

    self.assertDictEqual(c.to_dict(), {'fieldEnum': 'ONE'})

  def test_class_render_change_case(self) -> None:
    @dataclass_json
    @dataclass
    class ToTest(object):
      field_enum: TestEnum = enum_field(default=TestEnum.ONE,
                                        letter_case=LetterCase.SNAKE)
    c = ToTest()

    self.assertDictEqual(c.to_dict(), {'field_enum': 'ONE'})

  def test_class_render_change_name(self) -> None:
    @dataclass_json
    @dataclass
    class ToTest(object):
      field_enum: TestEnum = enum_field(default=TestEnum.ONE,
                                        field_name='foo')
    c = ToTest()

    self.assertDictEqual(c.to_dict(), {'foo': 'ONE'})


class MetadataTest(unittest.TestCase):
  def test_simple_update(self) -> None:
    base = {'pirate': 'Westley'}

    updated = metadata(base=base, pirate='Dread Pirate Roberts')

    self.assertDictEqual(updated, {'pirate': 'Dread Pirate Roberts'})

  def test_remove(self) -> None:
    base = {'pirate': 'Dread Pirate Roberts', 'swordsman': 'Inigo Montoya'}

    updated = metadata(base=base, swordsman=None)

    self.assertDictEqual(updated, {'pirate': 'Dread Pirate Roberts'})

  def test_add(self) -> None:
    base = {'pirate': 'Dread Pirate Roberts', }

    updated = metadata(base=base, swordsman='Inigo Montoya')

    self.assertDictEqual(updated, {'pirate': 'Dread Pirate Roberts',
                                   'swordsman': 'Inigo Montoya'})

  def test_add_and_remove(self) -> None:
    base = {'pirate': 'Dread Pirate Roberts', 'strongman': 'Fezzik'}

    updated = metadata(base=base, swordsman='Inigo Montoya', strongman=None)

    self.assertDictEqual(updated, {'pirate': 'Dread Pirate Roberts',
                                   'swordsman': 'Inigo Montoya'})

  def test_add_update_and_remove(self) -> None:
    base = {'pirate': 'Westley', 'strongman': 'Fezzik'}

    updated = metadata(base=base, swordsman='Inigo Montoya', strongman=None,
                       pirate='Dread Pirate Roberts')

    self.assertDictEqual(updated, {'pirate': 'Dread Pirate Roberts',
                                   'swordsman': 'Inigo Montoya'})



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass, field
from typing import Dict, Optional

from dataclasses_json import LetterCase, dataclass_json
from dataclasses_json import (DataClassJsonMixin, LetterCase, config)
from dataclasses_json.core import Json

from card_framework import standard_field

from ..widget import Widget
from .button import Button
from .icon import Icon
from .on_click import OnClick
from .switch_control import SwitchControl


@dataclass
class DecoratedText(Widget, DataClassJsonMixin):
  """DecoratedText

  This will render a DecoratedText widget.

  https://developers.google.com/chat/api/guides/message-formats/cards#decoratedtext
  """
  icon: Optional[Icon] = standard_field()
  start_icon: Optional[Icon] = standard_field()
  top_label: str = standard_field()
  text: str = standard_field()
  wrap_text: bool = standard_field()
  bottom_label: str = standard_field()
  on_click: Optional[OnClick] = standard_field()
  button: Optional[Button] = standard_field()
  switch_control: Optional[SwitchControl] = standard_field()
  end_icon: Optional[Icon] = standard_field()

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    return 'decoratedText'

  def to_dict(self, encode_json=False) -> Dict[str, Json]:
    """Converts the dataclass to a dict.

    This is an override of the standard dataclass `to_dict` method to allow
    validation that `action` and `open_link` are mutually exclusive.

    Args:
        encode_json (bool, optional): encode the json strings. Defaults to False.

    Raises:
        ValueError: if both `known_icon` and `icon_url` are set.

    Returns:
        Dict[str, Json]: The header
    """
    if sum(i is not None
           for i in [self.button, self.switch_control, self.end_icon]) > 1:
      raise ValueError(
          'Only one of [button, switch_control, end_icon] can be set.')

    return super().to_dict(encode_json)



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from dataclasses import dataclass
from typing import List

from dataclasses_json import LetterCase, dataclass_json

from card_framework.v2.enums import HorizontalAlignment

from .widget import Widget


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class InvalidWidget(Widget):
  """Bad Widget, no override"""


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ValidWidget(Widget):
  """Good Widget"""
  camel_case_property: str = 'Inigo Montoya'

  @property
  def _widget_tag(self) -> str:
    return 'widget'


class WidgetTest(unittest.TestCase):
  def test_invalid_widget(self) -> None:
    widget = InvalidWidget()
    with self.assertRaises(NotImplementedError):
      widget._widget_tag

  def test_valid_widget(self) -> None:
    widget = ValidWidget()
    self.assertEqual(widget._widget_tag, 'widget')

  def test_valid_widget_render(self) -> None:
    widget = ValidWidget()
    self.assertDictEqual(
        widget.render(),
        {'widget': {'camelCaseProperty': 'Inigo Montoya'}})



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass, field
from typing import Dict, Optional

from dataclasses_json import DataClassJsonMixin, LetterCase, config
from dataclasses_json.core import Json

from card_framework import enum_field, standard_field

from .enums import ImageType


@dataclass
class CardHeader(DataClassJsonMixin):
  """CardHeader

  Describes a Google Chat App response header.

  https://developers.google.com/chat/api/reference/rest/v1/cards#cardheader
  """
  title: Optional[str] = standard_field()
  subtitle: Optional[str] = standard_field()
  image_url: Optional[str] = standard_field()
  image_type: Optional[ImageType] = enum_field()
  image_alt_text: Optional[str] = standard_field()

  def to_dict(self, encode_json=False) -> Dict[str, Json]:
    """Converts the dataclass to a dict.

    This is an override of the standard dataclass `to_dict` method to allow
    validation that `image_style` is set if `image_url` is provided.

    Args:
        encode_json (bool, optional): encode the json strings. Defaults to False.

    Raises:
        ValueError: if the `image_style` is not set with the `image_url`

    Returns:
        Dict[str, Json]: The header
    """
    if self.image_url and not self.image_type:
      raise ValueError('If image_url is used, image_style must be set.')

    return super().to_dict(encode_json)



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

import dataclasses
from typing import Any, Iterable, Mapping

import dataclasses_json

from card_framework import enum_field, standard_field

from .dialog_action import DialogAction
from .enums import ResponseType


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL)
@dataclasses.dataclass
class ActionResponse(object):
  _tag = 'actionResponse'

  @property
  def tag(self) -> str:
    return self._tag

  @tag.setter
  def tag(self, value: str) -> None:
    self._tag = value

  type: ResponseType = enum_field()
  url: str = standard_field()
  dialog_action: DialogAction = standard_field()

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """

    return {self.tag: self.to_dict()}



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass, field
from typing import Any, Dict, Mapping, Optional

from card_framework import enum_field, standard_field


from ..enums import Icon, TextInputType
from dataclasses_json import DataClassJsonMixin, LetterCase, config, dataclass_json
from dataclasses_json.core import Json

from .action import Action
from .suggestions import Suggestions
from ..widget import Widget


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TextInput(Widget):
  """TextInput
  """
  name: str = standard_field()
  label: Optional[str] = standard_field()
  hint_text: Optional[str] = standard_field()
  value: Optional[str] = standard_field()
  type: TextInputType = enum_field()
  on_change_action: Optional[Action] = standard_field()
  initial_suggestions: Optional[Suggestions] = standard_field()
  auto_complete_action: Optional[Action] = standard_field()

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    return 'textInput'



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from card_framework.v2.widgets.action import Action, ActionParameter
from card_framework.v2.widgets.suggestions import SuggestionItem, Suggestions

from .text_input import TextInput


class TextInputTest(unittest.TestCase):
  def test_simple_render(self) -> None:
    self.assertDictEqual(TextInput(name='Inigo Montoya').render(),
                         {'textInput': {'name': 'Inigo Montoya'}})

  def test_render_complete(self) -> None:
    self.maxDiff = None
    i = TextInput()
    i.name = 'inigo'
    i.hint_text = 'You killed my father.'
    i.initial_suggestions = Suggestions(
        [SuggestionItem(text='Prepare to die.')]
    )
    i.label = 'Inigo Montoya'
    i.value = 'Anything you want.'
    i.auto_complete_action = Action(function='duel')
    i.on_change_action = Action(function='switch_hand', parameters=[
        ActionParameter(key='Over too quick', value='left'),
        ActionParameter(key='Not left handed', value='right')])

    print(i.render())

    self.assertDictEqual(i.render(),
                         {'textInput': {'name': 'inigo',
                                        'label': 'Inigo Montoya',
                                        'hintText': 'You killed my father.',
                                        'onChangeAction': {
                                            'function': 'switch_hand',
                                            'parameters': [
                                                {'key': 'Over too quick',
                                                 'value': 'left'},
                                                {'key': 'Not left handed',
                                                 'value': 'right'}
                                            ]},
                                        'value': 'Anything you want.',
                                        'initialSuggestions': {
                                            'items': [
                                                {'text': 'Prepare to die.'}
                                            ]},
                                        'autoCompleteAction': {
                                            'function': 'duel',
                                        }}}
                         )



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from ..enums import HorizontalAlignment, Icon as KnownIcon
from .button import Button
from .icon import Icon
from .on_click import OnClick
from .open_link import OpenLink


class ButtonTest(unittest.TestCase):
  def test_text_button(self) -> None:
    text = 'Inconceivable!'
    on_click = OnClick(open_link=OpenLink(url='https://www.karentaylorart.com'))
    t = Button(text=text, on_click=on_click)

    self.assertDictEqual(
        t.to_dict(),
        {
            'onClick': {'openLink': {'url': 'https://www.karentaylorart.com'}},
            'text': 'Inconceivable!'}
    )

  def test_icon_button(self) -> None:
    i = Button(on_click=OnClick(open_link='http://www.karentaylorart.com'))
    i.icon = Icon(known_icon=KnownIcon.AIRPLANE)
    self.assertDictEqual(
        i.to_dict(),
        {
            'icon': {
                'knownIcon': 'AIRPLANE'},
            'onClick': {'openLink': 'http://www.karentaylorart.com'},
        }
    )

  def test_icon_url_button(self) -> None:
    self.maxDiff = None
    i = Button(on_click=OnClick(open_link='http://www.karentaylorart.com'))
    i.icon = Icon(
        icon_url='https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg')
    self.assertDictEqual(
        i.to_dict(),
        {
            'icon': {
                'iconUrl': 'https://www.karentaylorart.com/wp-content/uploads/2021/01/go_small.jpg',
            },
            'onClick': {'openLink': 'http://www.karentaylorart.com'},
        })

  def test_aliugned_text_button(self) -> None:
    text = 'Inconceivable!'
    on_click = OnClick(open_link=OpenLink(url='https://www.karentaylorart.com'))
    t = Button(text=text, on_click=on_click)
    t.horizontal_alignment = HorizontalAlignment.CENTER

    self.assertDictEqual(
        t.to_dict(),
        {
            'onClick': {
                'openLink': {
                    'url': 'https://www.karentaylorart.com'}
            },
            'text': 'Inconceivable!'}
    )



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
  long_description = fh.read()

setuptools.setup(name='python-card-framework',
                 version='0.4.0',
                 description='API for rendering Chat App Card json.',
                 long_description=long_description,
                 long_description_content_type='text/markdown',
                 url='https://github.com/google/python-card-framework',
                 author='David Harcombe',
                 author_email='davidharcombe@google.com',
                 license='Apache 2.0',
                 zip_safe=False,
                 include_package_data=True,
                 packages=setuptools.find_packages(),
                 classifiers=[
                     'Development Status :: 4 - Beta',
                     'Intended Audience :: Developers',
                     'Topic :: Software Development',
                     'License :: OSI Approved :: Apache Software License',
                     'Programming Language :: Python :: 3',
                     'Programming Language :: Python :: 3.9',
                 ],
                 python_requires=">=3.10",
                 install_requires=[
                     'dataclasses-json>=0.5.2',
                     'stringcase>=1.2.0'
                 ],
                 )



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from dataclasses import dataclass, field
from typing import Any, Iterable, List, Mapping
from dataclasses_json import LetterCase, dataclass_json, config

from card_framework import list_field
from ..widget import Widget
from .button import Button


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ButtonList(Widget):
  """ButtonList

  This will render a ButtonList widget.

  https://developers.google.com/chat/api/guides/message-formats/cards#buttonlist
  """
  buttons: List[Button] = list_field()

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    return 'buttonList'

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return {self._widget_tag: self.to_dict()}



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
from dataclasses import dataclass
from typing import List

from dataclasses_json import LetterCase, dataclass_json

from card_framework.v2.annotation import *
from card_framework.v2.enums import UserType


class AnnotationTest(unittest.TestCase):
  def test_render_slash_command(self) -> None:
    a = Annotation(type=AnnotationType.SLASH_COMMAND)
    s = SlashCommandMetadata()
    s.type = SlashCommandMetadata.SlashCommandMetadataType.INVOKE
    s.command_id = 1
    s.command_name = '/tothepain'
    a.slash_command = s

    self.assertDictEqual(
        a.render(),
        {'annotation': {'slashCommand': {'commandId': 1,
                                         'commandName': '/tothepain',
                                         'type': 'INVOKE'},
                        'type': 'SLASH_COMMAND'}}
    )

  def test_render_user_mention(self) -> None:
    a = Annotation(type=AnnotationType.USER_MENTION)
    u = UserMentionMetadata()
    u.type = UserMentionMetadata.UserMentionMetadataType.MENTION
    user = User()
    user.type = UserType.HUMAN
    user.name = 'westley'
    user.display_name = 'Dread Pirate Roberts'
    u.user = user
    a.user_mention = u

    self.assertDictEqual(
        a.render(),
        {'annotation': {
            'type': 'USER_MENTION',
            'userMention': {
                'type': 'MENTION',
                'user': {
                    'displayName': 'Dread Pirate Roberts',
                    'name': 'westley',
                    'type': 'HUMAN'
                }
            }
        }
        })



# Copyright 2021 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     https://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations
import enum

# Definite v2


class AutoNumber(enum.Enum):
  def __repr__(self):
    return '<%s.%s>' % (self.__class__.__name__, self.name)

  def __new__(cls, *args, **kwargs):
    value = len(cls.__members__) + 1
    obj = object.__new__(cls)
    obj._value_ = value
    return obj


class LoadIndicator(AutoNumber):
  """LoadIndicator
  """
  SPINNER = 'SPINNER'
  NONE = 'NONE'


class DisplayStyle(AutoNumber):
  DISPLAY_STYLE_UNSPECIFIED = 'DISPLAY_STYLE_UNSPECIFIED'
  PEEK = 'PEEK'
  REPLACE = 'REPLACE'


class ControlType(AutoNumber):
  SWITCH = 'SWITCH'
  CHECK_BOX = 'CHECK_BOX'

  @classmethod
  def _missing_(cls, value) -> ControlType:
    """Backward compatilbility for old enums.

    If the old product names are still in use, replace them with
    the new values seamlessly.

    Args:
        value (str): enum string value requested

    Returns:
        ControlType: the corrected type

    Raises:
        ValueError if it was simply an incorrect enum rather an old value
    """
    if value == 'CHECKBOX':
      # Deprecated in favor of CHECK_BOX.
      return cls.CHECK_BOX


class ImageType(AutoNumber):
  """ImageType

  The possible image styles of a Header image.
  """
  SQUARE = 'SQUARE'
  CIRCLE = 'CIRCLE'


class SelectionInputType(AutoNumber):
  """SelectionInputType
  """
  SWITCH = 'SWITCH'
  CHECK_BOX = 'CHECK_BOX'
  RADIO_BUTTON = 'RADIO_BUTTON'
  DROPDOWN = 'DROPDOWN'

# Unknown


class ImageStyle(AutoNumber):
  """ImageStyle

  The possible image styles of a Header image.
  """
  AVATAR = 'AVATAR'
  IMAGE = 'IMAGE'


class Icon(AutoNumber):
  """Icon

  The allowed built in icon values.

  https://developers.google.com/chat/api/guides/message-formats/cards#builtinicons
  """
  AIRPLANE = 'AIRPLANE'
  BOOKMARK = 'BOOKMARK'
  BUS = 'BUS'
  CAR = 'CAR'
  CLOCK = 'CLOCK'
  CONFIRMATION_NUMBER_ICON = 'CONFIRMATION_NUMBER_ICON'
  DESCRIPTION = 'DESCRIPTION'
  DOLLAR = 'DOLLAR'
  EMAIL = 'EMAIL'
  EVENT_SEAT = 'EVENT_SEAT'
  FLIGHT_ARRIVAL = 'FLIGHT_ARRIVAL'
  FLIGHT_DEPARTURE = 'FLIGHT_DEPARTURE'
  HOTEL = 'HOTEL'
  HOTEL_ROOM_TYPE = 'HOTEL_ROOM_TYPE'
  INVITE = 'INVITE'
  MAP_PIN = 'MAP_PIN'
  MEMBERSHIP = 'MEMBERSHIP'
  MULTIPLE_PEOPLE = 'MULTIPLE_PEOPLE'
  PERSON = 'PERSON'
  PHONE = 'PHONE'
  RESTAURANT_ICON = 'RESTAURANT_ICON'
  SHOPPING_CART = 'SHOPPING_CART'
  STAR = 'STAR'
  STORE = 'STORE'
  TICKET = 'TICKET'
  TRAIN = 'TRAIN'
  VIDEO_CAMERA = 'VIDEO_CAMERA'
  VIDEO_PLAY = 'VIDEO_PLAY'


class HorizontalAlignment(AutoNumber):
  """HorizontalAlignment
  """
  START = 'START'
  CENTER = 'CENTER'
  END = 'END'

  def __str__(self) -> str:
    return self.name

  def __repr__(self) -> str:
    return str(self.name)


class TextInputType(AutoNumber):
  """TextInputType
  """
  SINGLE_LINE = 'SINGLE_LINE'
  MULTIPLE_LINE = 'MULTIPLE_LINE'


class OnClose(AutoNumber):
  """OnClose
  """
  NOTHING = 'NOTHING'
  RELOAD = 'RELOAD'


class OpenAs(AutoNumber):
  """OpenAs
  """
  FULL_SIZE = 'FULL_SIZE'
  OVERLAY = 'OVERLAY'


class Code(AutoNumber):
  OK = 'OK'
  CANCELLED = 'CANCELLED'
  UNKNOWN = 'UNKNOWN'
  INVALID_ARGUMENT = 'INVALID_ARGUMENT'
  DEADLINE_EXCEEDED = 'DEADLINE_EXCEEDED'
  NOT_FOUND = 'NOT_FOUND'
  ALREADY_EXISTS = 'ALREADY_EXISTS'
  PERMISSION_DENIED = 'PERMISSION_DENIED'
  UNAUTHENTICATED = 'UNAUTHENTICATED'
  RESOURCE_EXHAUSTED = 'RESOURCE_EXHAUSTED'
  FAILED_PRECONDITION = 'FAILED_PRECONDITION'
  ABORTED = 'ABORTED'
  OUT_OF_RANGE = 'OUT_OF_RANGE'
  UNIMPLEMENTED = 'UNIMPLEMENTED'
  INTERNAL = 'INTERNAL'
  UNAVAILABLE = 'UNAVAILABLE'
  DATA_LOSS = 'DATA_LOSS'


class ResponseType(AutoNumber):
  TYPE_UNSPECIFIED = 'TYPE_UNSPECIFIED'
  NEW_MESSAGE = 'NEW_MESSAGE'
  UPDATE_MESSAGE = 'UPDATE_MESSAGE'
  UPDATE_USER_MESSAGE_CARDS = 'UPDATE_USER_MESSAGE_CARDS'
  REQUEST_CONFIG = 'REQUEST_CONFIG'
  DIALOG = 'DIALOG'


class UserType(AutoNumber):
  TYPE_UNSPECIFIED = 'TYPE_UNSPECIFIED'
  HUMAN = 'HUMAN'
  BOT = 'BOT'


class AnnotationType(AutoNumber):
  ANNOTATION_TYPE_UNSPECIFIED = 'ANNOTATION_TYPE_UNSPECIFIED'
  USER_MENTION = 'USER_MENTION'
  SLASH_COMMAND = 'SLASH_COMMAND'


class SpaceType(AutoNumber):
  SPACE_TYPE_UNSPECIFIED = 'SPACE_TYPE_UNSPECIFIED'
  SPACE = 'SPACE'
  GROUP_CHAT = 'GROUP_CHAT'
  DIRECT_MESSAGE = 'DIRECT_MESSAGE'


class Source(AutoNumber):
  SOURCE_UNSPECIFIED = 'SOURCE_UNSPECIFIED'
  DRIVE_FILE = 'DRIVE_FILE'
  UPLOADED_CONTENT = 'UPLOADED_CONTENT'



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from .card_header import CardHeader
from .enums import ImageType


class HeaderTest(unittest.TestCase):
  def test_full_header(self) -> None:
    h = CardHeader(
        title='The Princess Bride',
        subtitle=("There's a shortage of perfect movies in this world. "
                  "It would be a pity to damage this one."),
        image_url='https://en.wikipedia.org/wiki/File:Princess_bride.jpg',
        image_alt_text="It's as real as the feelings you feel.",
        image_type=ImageType.CIRCLE)

    self.assertDictEqual(
        h.to_dict(),
        {
            'imageType': 'CIRCLE',
            'imageAltText': "It's as real as the feelings you feel.",
            'imageUrl': 'https://en.wikipedia.org/wiki/File:Princess_bride.jpg',
            'subtitle': "There's a shortage of perfect movies in this world. "
                        "It would be a pity to damage this one.",
            'title': 'The Princess Bride'})

  def test_header_no_image(self) -> None:
    h = CardHeader(
        title='The Princess Bride',
        subtitle=("There's a shortage of perfect movies in this world. "
                  "It would be a pity to damage this one."))

    self.assertDictEqual(h.to_dict(),
                         {'subtitle': "There's a shortage of perfect movies in this world. It would be "
                          'a pity to damage this one.',
                          'title': 'The Princess Bride',
                          })

  def test_header_no_subtitle(self) -> None:
    h = CardHeader(title='The Princess Bride')

    self.assertDictEqual(h.to_dict(),
                         {'title': 'The Princess Bride', })

  def test_header_image_no_style(self) -> None:
    h = CardHeader(
        title='The Princess Bride',
        image_url='https://en.wikipedia.org/wiki/File:Princess_bride.jpg',
        subtitle=("There's a shortage of perfect movies in this world. "
                  "It would be a pity to damage this one."))

    with self.assertRaises(ValueError):
      h.to_dict()



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import dataclasses
from typing import Any, Iterable, List, Mapping, Optional
import uuid

import dataclasses_json

from card_framework import list_field, standard_field

from .enums import DisplayStyle

from .card_header import CardHeader
from .section import Section
from .card_action import CardAction
from .card_fixed_footer import CardFixedFooter


@dataclasses_json.dataclass_json(letter_case=dataclasses_json.LetterCase.CAMEL,
                                 undefined=dataclasses_json.Undefined.EXCLUDE)
@dataclasses.dataclass
class Card(object):
  """Response

  A response object that can be `render`ed to produce a valid Google Chat App
  json response.

  See https://developers.google.com/chat/api/guides/message-formats/cards for
  full details on what this should look like.
  """
  _tag = 'cards'
  _card_id = None

  @property
  def tag(self) -> str:
    return self._tag

  @tag.setter
  def tag(self, value: str) -> None:
    self._tag = value

  @property
  def card_id(self) -> str:
    return self._card_id

  @card_id.setter
  def card_id(self, value: str) -> None:
    self._card_id = value

  header: Optional[CardHeader] = standard_field()
  name: Optional[str] = standard_field()
  sections: Optional[List[Section]] = list_field(default_factory=list)
  card_actions: Optional[List[CardAction]] = standard_field()
  fixed_footer: Optional[CardFixedFooter] = standard_field()
  display_style: Optional[DisplayStyle] = standard_field()
  peek_card_header: Optional[CardHeader] = standard_field()

  def add_section(self, section: Section) -> None:
    """Adds a section to the report.

    Args:
        section (Section): The section to add.
    """
    self.sections.append(section)

  def render(self, remove_empty_strings: bool = False) -> Mapping[str, Any]:
    """Renders the response to json.

    The `strip_nulls` internal function removes all `None` values from the dict
    before rendering it. Passing the additional `remove_empty_strings`
    parameter will also remove json keys with no text.

    Args:
      remove_empty_strings (bool): Also remove empty strings. Default is False.
    Returns:
        Mapping[str, Any]: _description_
    """
    def strip_nulls(value: Iterable) -> Iterable:
      """Removes null values from iterables.

      Recursively remove all None values from dictionaries and lists, and
      return the result as a new dictionary or list.

      Args:
        value (Any): any list or dict to have empty values removed.
      """
      if isinstance(value, list):
        return [strip_nulls(x) for x in value if x is not None]
      elif isinstance(value, dict):
        return {
            key: strip_nulls(val)
            for key, val in value.items() if val is not None and
            not (remove_empty_strings and val == '')
        }
      else:
        return value

    return strip_nulls({
        'cardId': self.card_id or str(uuid.uuid4()),
        'card': self.to_dict(),
    })



# Copyright 2022 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, List, Mapping

from card_framework import enum_field, list_field, standard_field
from dataclasses_json import LetterCase, config, dataclass_json

from ..enums import AutoNumber, HorizontalAlignment
from ..widget import Widget
from .color import Color
from .on_click import OnClick


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class Grid(Widget):
  title: str = standard_field()
  items: List[GridItem] = list_field()
  border_style: BorderStyle = standard_field()
  column_count: int = standard_field()
  on_click: OnClick = standard_field()

  @property
  def _widget_tag(self) -> str:
    """The widget tag name.

    Returns:
        str: The key by which the widget will be rendered in the Section.
    """
    return 'grid'

  def render(self) -> Mapping[str, Any]:
    """Renders the response to json.

    Returns:
        Mapping[str, Any]: _description_
    """
    return {self._widget_tag: self.to_dict(), }


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class GridItem(object):
  class GridItemLayout(AutoNumber):
    GRID_ITEM_LAYOUT_UNSPECIFIED = 'GRID_ITEM_LAYOUT_UNSPECIFIED'
    TEXT_BELOW = 'TEXT_BELOW'
    TEXT_ABOVE = 'TEXT_ABOVE'

  id: str = standard_field()
  image: ImageComponent = standard_field()
  title: str = standard_field()
  subtitle: str = standard_field()
  text_alignment: HorizontalAlignment = enum_field()
  layout: GridItemLayout = enum_field()


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ImageComponent(object):
  image_uri: str = standard_field()
  alt_text: str = standard_field()
  crop_style: ImageCropStyle = standard_field()
  border_style: BorderStyle = standard_field()


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class ImageCropStyle(object):
  class ImageCropType(AutoNumber):
    IMAGE_CROP_TYPE_UNSPECIFIED = 'IMAGE_CROP_TYPE_UNSPECIFIED'
    SQUARE = 'SQUARE'
    CIRCLE = 'CIRCLE'
    RECTANGLE_CUSTOM = 'RECTANGLE_CUSTOM'
    RECTANGLE_4_3 = 'RECTANGLE_4_3'

  type: ImageCropType = enum_field()
  aspect_ratio: float = standard_field()


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class BorderStyle(object):
  class BorderType(AutoNumber):
    BORDER_TYPE_UNSPECIFIED = 'BORDER_TYPE_UNSPECIFIED'
    NO_BORDER = 'NO_BORDER'
    STROKE = 'STROKE'

  type: BorderType = enum_field()
  stroke_color: Color = standard_field()
  corner_radius: int = standard_field()
